package org.apache.xindice.client.xmldb.xmlrpc;
/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: DatabaseImpl.java,v 1.7 2002/11/21 07:44:35 vladimir Exp $
 */
import org.apache.xindice.client.xmldb.CommonConfigurable;

import org.xmldb.api.base.Collection;
import org.xmldb.api.base.Database;
import org.xmldb.api.base.ErrorCodes;
import org.xmldb.api.base.XMLDBException;

/**
 * implements XML:DB's <code>Database</code> interface using XML-RPC to
 * communicate with the Xindice server.
 *
 * Note this class is a database <em>driver</em>, and one class of this database
 * could be used to connect to <em>many</em> different databases.
 *
 * @author <a href="mailto:james.bates@amplexor.com">James Bates</a>
 */
public class DatabaseImpl extends CommonConfigurable implements Database {
    
    /* prefix used to denote XML:DB URI's that should use this driver */
    static String DRIVER_NAME = "xindice";
    
    /* XML:DB conformance level of this driver */
    private String CONFORMANCE_LEVEL = "0";
    
   /**
    * Determines whether this <code>Database</code> implementation  can handle
    * the URI. It should return true if the Database instance knows how to
    * handle the URI and false otherwise.
    *
    * @param uri the URI to check for.
    * @return true if the URI can be handled, false otherwise.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    *  <code>ErrroCodes.INVALID_URI</code> If the URI is not in a valid format. <br />
    */
    public boolean acceptsURI(String uri) throws XMLDBException {

        return ((uri != null) && uri.startsWith(getName() + "://"));
    }
    
   /**
    * Retrieves a <code>Collection</code> instance based on the URI provided
    * in the <code>uri</code> parameter. The format of the URI is defined in the
    * documentation for DatabaseManager.getCollection().<p/>
    *
    * Authentication is handled via username and password however it is not
    * required that the database support authentication. Databases that do not
    * support authentication MUST ignore the
    * <code>username</code> and <code>password</code> if those provided are not
    * null.
    *
    * @param uri the URI to use to locate the collection.
    * @param password The password to use for authentication to the database or
    *    null if the database does not support authentication.
    * @return A <code>Collection</code> instance for the requested collection or
    *  null if the collection could not be found.
    * @return The <code>Collection</code> instance
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    *  <code>ErrroCodes.INVALID_URI</code> If the URI is not in a valid format. <br />
    *  <code>ErrroCodes.PERMISSION_DENIED</code> If the <code>username</code>
    *    and <code>password</code> were not accepted by the database.
    */
    public Collection getCollection(String uri, String userName, String password)
            throws XMLDBException {
                
        /* TODO: introduce authentication some day */
                
        if (!acceptsURI(uri)) {
            
            throw new XMLDBException(ErrorCodes.INVALID_URI);
        }

        /* Chop off driver prefix, and '://' */
        uri = uri.substring(getName().length() + 3);
        
        /* Extract host name & port, if present */
        int firstSlash = uri.indexOf('/');
        if (firstSlash == -1) {
            
            throw new XMLDBException(ErrorCodes.INVALID_URI);
        }
        
        String hostPort = uri.substring(0, firstSlash);
        String collPath = uri.substring(firstSlash);

        /* Absent host defaults to localhost and standard Xindice HTTP port */
        if (hostPort.equals("")) {

            hostPort = "127.0.0.1:8080";
        }
        
		try {
        return new CollectionImpl(hostPort, collPath);
		}
		catch(XMLDBException e) {
           if(e.errorCode == ErrorCodes.NO_SUCH_COLLECTION) {
              // per getCollection contract, return null if not found
              return null;
           }
		   throw e;
		}
    }
    
    /**
     * Returns the prefix used in XML:DB to denote URI's that this driver can
     * handle.
     *
     * @return the prefix driver name
     * @exception XMLDBException with expected error codes.<br />
     *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
     *  specific errors that occur.<br />
     */
    public String getName() throws XMLDBException {

        return DRIVER_NAME;
    }
        
   /**
    * Returns the XML:DB API Conformance level for the implementation. This can
    * be used by client programs to determine what functionality is available to
    * them.
    *
    * @return the XML:DB API conformance level for this implementation.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
    public String getConformanceLevel() throws XMLDBException {

        return CONFORMANCE_LEVEL;
    }
    
}
