package org.apache.xindice.tools.command;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: ImportTree.java,v 1.7 2002/11/21 07:44:36 vladimir Exp $
 */

import org.apache.xindice.tools.XMLTools;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;

import java.io.File;
import java.io.FileFilter;
import java.util.Hashtable;

/**
 * ImportTree.java is designed to take a directory/file path and create a Collection
 * from it's name. The files within the directory are converted into Documents
 * and then stored into the newly created Collection with their filename as a document key
 *
 * NOTE:  If the collection passed in with the -c option already exists it will be overwritten!
 */

public class ImportTree extends Command {
   Command addDocument = null;
   Command addCollection = null;
   
   public boolean execute(Hashtable table) throws Exception {
      Collection col = null;
      try {
         // Setup the creation commands we'll use to add documents and create
         // collections.
         addDocument =
            (Command) Class.forName("org.apache.xindice.tools.command.AddDocument").newInstance();
   
         addCollection =
            (Command) Class.forName("org.apache.xindice.tools.command.AddCollection").newInstance();
            
         String startcollection = null;

         // Verify that the correct command line parameters were passed in
         if (table.get(XMLTools.FILE_PATH) == null) {
            System.out.println("ERROR : Directory name and switch required");
            return false;
         }
         if (table.get(XMLTools.COLLECTION) == null) {
            System.out.println("ERROR : Collection name and switch required");
            return false;
         }
         
         startcollection = (String) table.get(XMLTools.COLLECTION);
         
         // Make sure the root collection exists
         String colstring = normalizeCollectionURI(startcollection,
                                                   (String) table.get(XMLTools.LOCAL));
         if ((col = DatabaseManager.getCollection(colstring)) == null) {
            System.out.println("ERROR : Collection not found!");
            return false;
         }

         File startdir = new File((String) table.get(XMLTools.FILE_PATH));

         // Make call to process, this is called recursively!
         process(startdir, startcollection, table);

      } finally {
         // Be sure to close collection objects
         if (col != null) {
            col.close();
         }
      }

      return true;
   }

   // Recursive function to create collections/documents
   void process(File directory, String baseCollection, Hashtable table)
         throws Exception {
     
      try {
         // The file extension to use for reading in files
         final String ext = (String) table.get(XMLTools.EXTENSION);

         // Make sure we received a directory so we don't create a collection
         // named after a file.
         if ( ! directory.isDirectory() ) {
            System.out.println("ERROR: The specified import path is not a directory " +
                  directory.getPath());
            return;
         }

         // Create the collection for this directory.
         baseCollection = createCollection(baseCollection, directory.getName());
         
         // Get a directory listing and loop over
         File[] subdirs = directory.listFiles(new ExtensionFileFilter(ext));

         for (int i = 0; i < subdirs.length; i++) {
            // Is this a directory?  If so, call ourselves recursively
            if (subdirs[i].isDirectory()) {
               process(subdirs[i], baseCollection, table);
            }
            else {
               // Otherwise we have a file so import it.
               importFile(baseCollection, subdirs[i].getAbsolutePath(),
                  subdirs[i].getName());
            }
         }
      }
      catch (Exception e) {
         System.out.println("ERROR : " + e.getMessage());
         if (table.get(XMLTools.VERBOSE).equals("true")) {
            e.printStackTrace(System.err);
         }
         return;
      }
   }
      
   protected void importFile(String baseCollection, String path, String name)
         throws Exception {
      Hashtable table = new Hashtable();
      // Use functionality from AddDocument to add this document
      table.put(XMLTools.COLLECTION, baseCollection);
      table.put(XMLTools.FILE_PATH, path);
      table.put(XMLTools.NAME_OF, name);
            
      addDocument.execute(table);
   }
   
   protected String createCollection(String baseCollection, String newCollection)
         throws Exception {
      if (! newCollection.equals(".")) {
         Hashtable table = new Hashtable();
         table.put(XMLTools.COLLECTION, baseCollection);
         table.put(XMLTools.NAME_OF, newCollection);
                  
         addCollection.execute(table);
      }
      else {
         newCollection = "";
      }
      
      return baseCollection + "/" + newCollection;
   }

   /**
    * Allow us to filter directory listings over certain file extensions
    */
   private static class ExtensionFileFilter implements FileFilter {

      private String extension = "";

      /**
       * Constructs an <code>MyFileFiler</code> instance with a specific
       * extension.
       *
       * @param extension The file's extension
       */
      public ExtensionFileFilter(String extension) {
         this.extension = extension;
      }

      /**
       * Tests whether or not the specified abstract pathname should be
       * included in a pathname list.
       *
       * @param  pathname  The abstract pathname to be tested
       * @return  <code>true</code> if and only if <code>pathname</code>
       *          should be included
       */
      public boolean accept(File pathname) {
         if (this.extension != "") {
            return pathname.getName().endsWith("." + this.extension) || pathname.isDirectory();
         }
         else {
            return true;
         }
      }
   }

}
